<?php

namespace Drupal\webserver_auth\StackMiddleware;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\HttpKernelInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Core\Session\SessionConfigurationInterface;
use Drupal\webserver_auth\WebserverAuthHelper;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\PageCache\RequestPolicyInterface;
use Drupal\Core\PageCache\ChainRequestPolicyInterface;
use Drupal\webserver_auth\PageCache\RequestPolicy\RemoteUserIsSet;


/**
 * Maintenance mode subscriber to log out users.
 */
class WebserverAuthMiddleware implements HttpKernelInterface {

  /**
   * The decorated kernel.
   *
   * @var \Symfony\Component\HttpKernel\HttpKernelInterface
   */
  protected $httpKernel;

  /**
   * The session configuration.
   *
   * @var \Drupal\Core\Session\SessionConfigurationInterface
   */
  protected $sessionConfiguration;

  /**
   * Helper class that brings some helper functionality related to webserver authentication.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $helper;

  /**
   * A policy rule determining the cacheability of a request.
   *
   * @var \Drupal\Core\PageCache\RequestPolicyInterface
   */
  protected $requestPolicy;

  /**
   * Module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Constructs a WebserverAuthMiddleware object.
   *
   * @param \Symfony\Component\HttpKernel\HttpKernelInterface $http_kernel
   *   The decorated kernel.
   *
   * @param \Drupal\Core\Session\SessionConfigurationInterface $session_configuration
   *   The session configuration.
   *
   * @param \Drupal\webserver_auth\WebserverAuthHelper $helper
   *   Helper class that brings some functionality related to webserver authentication.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *  Module handler service.
   *
   * @param \Drupal\Core\PageCache\RequestPolicyInterface $request_policy
   *   A policy rule determining the cacheability of a request.
   */
  public function __construct(HttpKernelInterface $http_kernel, SessionConfigurationInterface $session_configuration, WebserverAuthHelper $helper, ModuleHandlerInterface $module_handler, RequestPolicyInterface $request_policy) {
    $this->httpKernel = $http_kernel;
    $this->sessionConfiguration = $session_configuration;
    $this->helper = $helper;
    $this->moduleHandler = $module_handler;
    $this->requestPolicy = $request_policy;
  }

  /**
   * {@inheritdoc}
   */
  public function handle(Request $request, $type = self::MASTER_REQUEST, $catch = TRUE) {
    /**
     * We assuming that user should be logged in if remote user exists and valid.
     * In this case we need to prevent any page to be loaded from cache (to be able to
     * affect on Authentication and log in remote user).
     *
     * We only need this code if Drupal page_cache module is enabled.
     */
    $authname = $this->helper->getRemoteUser($request);
    if (!$authname) {
      // Depends on settings we'll allow/block access to the site
      // for anonymous users at all.
      $config = \Drupal::config('webserver_auth.settings');

      if ($config->get('block_access_to_the_site')) {

        // If redirect URL is set - redirecting user.
        if ($url = $config->get('block_access_redirect_url')) {
          // In our case we don't want response to be generated by controllers,
          // so just returning RedirectResponse based on provided URL.
          $response = new RedirectResponse($url);
          return $response;
        }
        else {
          exit(t('Access Denied'));
        }
      }
    }

    // Adding our own cache policy to prevent page from being cached if needed.
    if (isset($this->requestPolicy) && $this->requestPolicy instanceof ChainRequestPolicyInterface) {
      $this->requestPolicy->addPolicy(new RemoteUserIsSet($this->helper));
    }

    return $this->httpKernel->handle($request, $type, $catch);
  }
}
